<?php
/**
 * A file i/o iterator
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class FileIoIterator extends BaseIterator implements SeekableWritableFileIterator {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$pointerInstance	An instance of a InputOutputPointer class
	 * @return	$iteratorInstance	An instance of a Iterator class
	 */
	public final static function createFileIoIterator (InputOutputPointer $pointerInstance) {
		// Get new instance
		$iteratorInstance = new FileIoIterator();

		// Set the instance here
		$iteratorInstance->setPointerInstance($pointerInstance);

		// Return the prepared instance
		return $iteratorInstance;
	}

	/**
	 * Gets currently read data
	 *
	 * @return	$current	Currently read data
	 */
	public function current () {
		// Call pointer instance
		return $this->getPointerInstance()->current();
	}

	/**
	 * Gets current seek position ("key").
	 *
	 * @return	$key	Current key in iteration
	 */
	public function key () {
		// Return it
		return $this->getPointerInstance()->determineSeekPosition();
	}

	/**
	 * Advances to next "block" of bytes
	 *
	 * @return	void
	 */
	public function next () {
		// Call pointer instance
		$this->getPointerInstance()->next();
	}

	/**
	 * Rewinds to the beginning of the file
	 *
	 * @return	$status		Status of this operation
	 */
	public function rewind () {
		// Call pointer instance
		return $this->getPointerInstance()->rewind();
	}

	/**
	 * Checks wether the current entry is valid (not at the end of the file).
	 * This method will return TRUE if an emptied (nulled) entry has been found.
	 *
	 * @return	$isValid	Whether the next entry is valid
	 */
	public function valid () {
		// Call pointer instance
		return $this->getPointerInstance()->valid();
	}

	/**
	 * Seeks to given position
	 *
	 * @param	$seekPosition	Seek position in file
	 * @return	$status			Status of this operation
	 */
	public function seek ($seekPosition) {
		// Call pointer instance
		return $this->getPointerInstance()->seek($seekPosition);
	}

	/**
	 * Writes at given position by seeking to it.
	 *
	 * @param	$seekPosition	Seek position in file
	 * @param	$data			Data to be written
	 * @return	void
	 */
	public function writeAtPosition ($seekPosition, $data) {
		// First seek to it
		$this->seek($seekPosition);

		// Then write the data at that position
		$this->getPointerInstance()->writeToFile($data);
	}

	/**
	 * Size of file stack
	 *
	 * @return	$size	Size (in bytes) of file
	 */
	public function size () {
		// Call the pointer object
		$size = $this->getPointerInstance()->size();

		// Return result
		return $size;
	}

	/**
	 * Reads given amount of bytes from file.
	 *
	 * @param	$bytes	Amount of bytes to read
	 * @return	$data	Data read from file
	 */
	public function read ($bytes) {
		// Call pointer instance
		return $this->getPointerInstance()->read($bytes);
	}

	/**
	 * Analyzes entries in index file. This will count all found (and valid)
	 * entries, mark invalid as damaged and count gaps ("fragmentation"). If
	 * only gaps are found, the file is considered as "virgin" (no entries).
	 *
	 * @return	void
	 */
	public function analyzeFile () {
		// Just call the pointer instance
		$this->getPointerInstance()->analyzeFile();
	}
}

// [EOF]
?>
